@@ -9,22 +9,40 @@ from io import StringIO
 from typing import List
 
 
+def str2bool_plus(string: str, numeric: bool = False):
+    "Takes boolean'ish or numeric string, converts to bool or int"
+    if string.isnumeric():
+        number = int(string)
+        if numeric:
+            return number
+        return number > 0
+    elif not string or string.lower().capitalize() in (
+        "No",
+        "False",
+        "N",
+    ):
+        if numeric:
+            return 0
+        return False
+    elif string.lower().capitalize() in ("Yes", "True", "Y"):
+        if numeric:
+            return 1
+        return True
+    else:
+        raise ValueError(f'Expected boolean or numeric or empty value, got "{string}"')
+
+
 class DebugFlag:
     "Checks for DEBUG env value and holds 'debug' boolean, 'warning' string"
 
-    debug = os.getenv("DEBUG", "0")
+    debug_raw = os.getenv("DEBUG", "0")
     warning = None
-    if debug.isnumeric():
-        debug = int(debug) > 0
-    elif not debug:
-        debug = False
-    elif debug.lower().capitalize() in ("Yes", "True", "Y"):
-        debug = True
-    elif not debug or debug.lower().capitalize() in ("No", "False", "N"):
-        debug = False
-    else:
-        warning = f'Expected boolean or numeric or empty value for DEBUG, got "{debug}", assuming False'
+    try:
+        debug = str2bool_plus(debug_raw)
+    except ValueError:
+        warning = f'Expected boolean or numeric or empty value for DEBUG, got "{debug_raw}", assuming False'
         debug = False
+    del debug_raw
 
 
 class LogFlag:
@@ -36,6 +54,13 @@ class LogFlag:
     prefix = False
 
 
+class NoStdOutFlag:
+    "Holds global state for inhibiting stdout printing"
+
+    nostdout = False
+    nowarn = False
+
+
 class Styles:
     "Terminal control characters for color and style"
 
@@ -100,8 +125,11 @@ def print_normal(*what, **how):
     optional 'log': False
     """
     log = how.pop("log", LogFlag.log)
+    nostdout = how.pop("nostdout", NoStdOutFlag.nostdout)
+    file = how.get("file", sys.stdout)
 
-    print(*what, **how, flush=True)
+    if not (nostdout and file == sys.stdout):
+        print(*what, **how, flush=True)
 
     if log:
         # print to fake file before printing to log
@@ -129,12 +157,14 @@ def print_fancy(*what, **how):
     log = how.pop("log", LogFlag.log)
     loglevel = how.pop("loglevel", 5)
     logprefix = how.pop("logprefix", LogFlag.prefix)
+    nostdout = how.pop("nostdout", NoStdOutFlag.nostdout)
 
     # print colored text for interactive output
     if file.isatty():
-        print(color, end="", file=file, flush=True)
-        print(*what, **how, file=file, flush=True)
-        print(Styles.reset, end="", file=file, flush=True)
+        if not (nostdout and file == sys.stdout):
+            print(color, end="", file=file, flush=True)
+            print(*what, **how, file=file, flush=True)
+            print(Styles.reset, end="", file=file, flush=True)
     # print lines prefixed with loglevel for journal
     elif logprefix:
         # print to fake file, add line prefixes, print for real
@@ -166,6 +196,9 @@ def print_fancy(*what, **how):
 
     if notify and (not file.isatty() or notify == 2):
         try:
+            # TODO: revisit import placement
+            from uwsm.dbus import DbusInteractions
+
             bus_session = DbusInteractions("session")
             msg = str(*what)
             bus_session.notify(summary="Message", body=msg, urgency=notify_urgency)
@@ -221,6 +254,8 @@ def print_warning(*what, **how):
     log = how.pop("log", LogFlag.log)
     loglevel = how.pop("loglevel", 4)
     logprefix = how.pop("logprefix", LogFlag.prefix)
+    # special handling for warnings
+    nostdout = how.pop("nostdout", NoStdOutFlag.nostdout and NoStdOutFlag.nowarn)
 
     print_fancy(
         *what,
@@ -232,6 +267,7 @@ def print_warning(*what, **how):
         log=log,
         loglevel=loglevel,
         logprefix=logprefix,
+        nostdout=nostdout,
     )
 
 
@@ -293,6 +329,7 @@ if DebugFlag.debug:
             log=log,
             loglevel=loglevel,
             logprefix=logprefix,
+            nostdout=False,
         )
 
 else:
@@ -300,13 +337,3 @@ else:
     def print_debug(*what, **how):
         "Does nothing"
         pass
-
-
-def print_style(stls, *what, **how):
-    "Prints selected style(s), then args, then resets"
-    if isinstance(stls, str):
-        stls = [stls]
-    for style in stls:
-        print(style, end="", flush=True)
-    print(*what, **how, flush=True)
-    print(Styles.reset, end="", file=sys.stderr, flush=True)
